@@ -1,7 +1,7 @@
-"""unittest/unittest2 compatibilty wrapper.
+"""unittest/unittest2 compatibility wrapper.
 
-Anything internal to nose2 *must* import unittest from here, to be
-sure that it is using unittest2 when on older pythons.
+Anything internal to nose2 *must* import unittest from here to be
+certain that it is using unittest2 when run on older Python versions.
 
 Yes::
 
@@ -26,7 +26,7 @@ try:
     unittest.installHandler
 except AttributeError:
     raise ImportError(
-        "Built-in unittest version too old, unittest2 is required")
+        "Built-in unittest version too old; unittest2 is required")
 
 __unittest = True
 
