@@ -23,10 +23,10 @@ test loaders to use:
 
    Plugins can use this hook to load tests from a class that is not a
    :class:`unittest.TestCase` subclass. To prevent other plugins from
-   loading tests from the test class, set ``event.handled`` to True and
+   loading tests from the test class, set ``event.handled`` to ``True`` and
    return a test suite. Plugins can also append tests to
-   ``event.extraTests`` -- ususally that's what you want to do, since
-   that will allow other plugins to load their tests from the test
+   ``event.extraTests``. Ususally, that's what you want, since
+   it allows other plugins to load their tests from the test
    case as well.
 
 .. function :: getTestMethodNames(self, event)
@@ -38,7 +38,7 @@ test loaders to use:
    :class:`unittest.TestCase` subclass by the standard nose2 test
    loader plugins (and other plugins that respect the results of the
    hook). To force a specific list of names, set ``event.handled`` to
-   True and return a list: this exact list will be the only test case
+   ``True`` and return a list: this exact list will be the only test case
    names loaded from the test case. Plugins can also extend the list
    of names by appending test names to ``event.extraNames``, and
    exclude names by appending test names to ``event.excludedNames``.
@@ -70,6 +70,7 @@ Here's an example of a test class::
 """
 
 import unittest
+import sys
 
 from nose2 import events, util
 from nose2.compat import unittest as ut2
@@ -86,7 +87,7 @@ class TestClassLoader(events.Plugin):
     def registerInSubprocess(self, event):
         event.pluginClasses.append(self.__class__)
 
-    def pluginsLoaded(self, event):
+    def register(self):
         """Install extra hooks
 
         Adds the new plugin hooks:
@@ -95,6 +96,7 @@ class TestClassLoader(events.Plugin):
         - getTestMethodNames
 
         """
+        super(TestClassLoader, self).register()
         self.addMethods('loadTestsFromTestClass', 'getTestMethodNames')
 
     def loadTestsFromModule(self, event):
@@ -102,9 +104,9 @@ class TestClassLoader(events.Plugin):
         module = event.module
         for name in dir(module):
             obj = getattr(module, name)
-            if (isinstance(obj, type) and not
-                issubclass(obj, unittest.TestCase) and not
-                issubclass(obj, unittest.TestSuite) and
+            if (isinstance(obj, type) and
+                not issubclass(obj, unittest.TestCase) and
+                not issubclass(obj, unittest.TestSuite) and
                 name.lower().startswith(self.session.testMethodPrefix)):
                 event.extraTests.append(
                     self._loadTestsFromTestClass(event, obj))
@@ -125,9 +127,9 @@ class TestClassLoader(events.Plugin):
             # name is a test case class
             event.extraTests.append(self._loadTestsFromTestClass(event, obj))
         elif (isinstance(parent, type) and
-              not issubclass(parent, unittest.TestCase) and not
-              util.isgenerator(obj) and not
-              hasattr(obj, 'paramList')):
+              not issubclass(parent, unittest.TestCase) and
+              not util.isgenerator(obj) and
+              not hasattr(obj, 'paramList')):
             # name is a single test method
             event.extraTests.append(
                 util.transplant_class(
@@ -141,10 +143,15 @@ class TestClassLoader(events.Plugin):
             loaded_suite = result or event.loader.suiteClass()
         else:
             names = self._getTestMethodNames(event, cls)
-            loaded_suite = event.loader.suiteClass(
-                [util.transplant_class(
-                 MethodTestCase(cls), cls.__module__)(name)
-                    for name in names])
+            try:
+                loaded_suite = event.loader.suiteClass(
+                    [util.transplant_class(
+                     MethodTestCase(cls), cls.__module__)(name)
+                        for name in names])
+            except:
+                _, ev, _ = sys.exc_info()
+                return event.loader.suiteClass(
+                    event.loader.failedLoadTests(cls.__name__, ev))
         if evt.extraTests:
             loaded_suite.addTests(evt.extraTests)
         # ... add extra tests
