@@ -11,19 +11,21 @@ __unittest = True
 
 
 class PluggableTestResult(object):
+
     """Test result that defers to plugins.
 
     All test outcome recording and reporting is deferred to plugins,
-    which are expected to implement startTest, stopTest, testOutcome,
-    and wasSuccessful.
+    which are expected to implement :func:`startTest`, :func:`stopTest`, 
+    :func:`testOutcome`, and :func:`wasSuccessful`.
 
     :param session: Test run session.
 
     .. attribute :: shouldStop
 
-       When True, test run should stop before running another test.
+       When ``True``, test run should stop before running another test.
 
     """
+
     def __init__(self, session):
         self.session = session
         self.shouldStop = False
@@ -109,10 +111,10 @@ class PluggableTestResult(object):
     def wasSuccessful(self):
         """Was test run successful?
 
-        Fires :func:`wasSuccessful` hook, returns ``event.success``.
+        Fires :func:`wasSuccessful` hook, and returns ``event.success``.
 
         """
-        # assume failure, plugins must affirmatively declare success
+        # assume failure; plugins must explicitly declare success
         try:
             return self._success
         except AttributeError:
@@ -124,7 +126,7 @@ class PluggableTestResult(object):
     def stop(self):
         """Stop test run.
 
-        Fires :func:`resultStop` hook, sets ``self.shouldStop`` to
+        Fires :func:`resultStop` hook, and sets ``self.shouldStop`` to
         ``event.shouldStop``.
 
         """
