@@ -1,3 +1,4 @@
+import sys
 import inspect
 import logging
 
@@ -21,30 +22,28 @@ class LayerSuite(unittest.BaseTestSuite):
         self.wasSetup = False
 
     def run(self, result):
-        self.setUp()
+        if not self._safeMethodCall(self.setUp, result):
+            return
         try:
             for test in self:
                 if result.shouldStop:
                     break
-                self.setUpTest(test)
+                self._safeMethodCall(self.setUpTest, result, test)
                 try:
                     test(result)
                 finally:
-                    self.tearDownTest(test)
+                    self._safeMethodCall(self.tearDownTest, result, test)
         finally:
             if self.wasSetup:
-                self.tearDown()
+                self._safeMethodCall(self.tearDown, result)
 
     def setUp(self):
-        # FIXME hook call
-        log.debug('in setUp layer %s', self.layer)
         if self.layer is None:
             return
 
         setup = self._getBoundClassmethod(self.layer, 'setUp')
         if setup:
             setup()
-            log.debug('setUp layer %s called', self.layer)
         self.wasSetup = True
 
     def setUpTest(self, test):
@@ -83,7 +82,16 @@ class LayerSuite(unittest.BaseTestSuite):
         teardown = self._getBoundClassmethod(self.layer, 'tearDown')
         if teardown:
             teardown()
-            log.debug('tearDown layer %s called', self.layer)
+
+    def _safeMethodCall(self, method, result, *args):
+        try:
+            method(*args)
+            return True
+        except KeyboardInterrupt:
+            raise
+        except:
+            result.addError(self, sys.exc_info())
+            return False
 
     def _allLayers(self, test, method, reverse=False):
         done = set()
@@ -108,8 +116,8 @@ class LayerSuite(unittest.BaseTestSuite):
 
     def _getBoundClassmethod(self, cls, method):
         """
-        Use instead of getattr to get only classmethods explicitly defined
-        on cls (not methods inherited from ancestors)
+        Use instead of :func:`getattr` to get only classmethods explicitly defined
+        on ``cls`` (not methods inherited from ancestors)
         """
         descriptor = cls.__dict__.get(method, None)
         if descriptor:
