@@ -2,6 +2,7 @@
 import os.path
 import tempfile
 import shutil
+import subprocess
 import sys
 
 import six
@@ -18,7 +19,7 @@ class TestCase(unittest.TestCase):
 
     """TestCase extension.
 
-    If the class variable _RUN_IN_TEMP is True (default: False), tests will be
+    If the class variable ``_RUN_IN_TEMP`` is ``True`` (default: ``False``), tests will be
     performed in a temporary directory, which is deleted afterwards.
     """
     _RUN_IN_TEMP = False
@@ -40,6 +41,32 @@ class TestCase(unittest.TestCase):
             os.chdir(self._orig_dir)
             shutil.rmtree(self._work_dir, ignore_errors=True)
 
+    def __str__(self):
+        """
+        In python 3.5, the unittest.TestCase.__str__() output changed.
+        This makes it conform to previous version.
+        """
+        if sys.version_info >= (3, 5):
+            test_module = self.__class__.__module__
+            test_class = self.__class__.__name__
+            test_method = self._testMethodName
+            return "%s (%s.%s)" % (test_method, test_module, test_class)
+        else:
+            return super(TestCase, self).__str__()
+
+    def id(self):
+        """
+        In python 3.5, the unittest.TestCase.__id__() output changed.
+        This makes it conform to previous version.
+        """
+        if sys.version_info >= (3, 5):
+            test_module = self.__class__.__module__
+            test_class = self.__class__.__name__
+            test_method = self._testMethodName
+            return "%s.%s.%s" % (test_module, test_class, test_method)
+        else:
+            return super(TestCase, self).id()
+
 
 class FunctionalTestCase(unittest.TestCase):
     tags = ['functional']
@@ -65,10 +92,13 @@ class FunctionalTestCase(unittest.TestCase):
     def runIn(self, testdir, *args, **kw):
         return run_nose2(*args, cwd=testdir, **kw)
 
+    def runModuleAsMain(self, testmodule, *args):
+        return run_module_as_main(testmodule, *args)
+
 
 class _FakeEventBase(object):
 
-    """Baseclass for fake Events."""
+    """Baseclass for fake :class:`~nose2.events.Event`\s."""
 
     def __init__(self):
         self.handled = False
@@ -91,7 +121,7 @@ class FakeHandleFileEvent(_FakeEventBase):
 
 class FakeStartTestEvent(_FakeEventBase):
 
-    """Fake StartTestEvent."""
+    """Fake :class:`~nose2.events.StartTestEvent`."""
 
     def __init__(self, test):
         super(FakeStartTestEvent, self).__init__()
@@ -103,7 +133,7 @@ class FakeStartTestEvent(_FakeEventBase):
 
 class FakeLoadFromNameEvent(_FakeEventBase):
 
-    """Fake LoadFromNameEvent."""
+    """Fake :class:`~nose2.events.LoadFromNameEvent`."""
 
     def __init__(self, name):
         super(FakeLoadFromNameEvent, self).__init__()
@@ -112,7 +142,7 @@ class FakeLoadFromNameEvent(_FakeEventBase):
 
 class FakeLoadFromNamesEvent(_FakeEventBase):
 
-    """Fake LoadFromNamesEvent."""
+    """Fake :class:`~nose2.events.LoadFromNamesEvent`."""
 
     def __init__(self, names):
         super(FakeLoadFromNamesEvent, self).__init__()
@@ -121,7 +151,7 @@ class FakeLoadFromNamesEvent(_FakeEventBase):
 
 class FakeStartTestRunEvent(_FakeEventBase):
 
-    """Fake StartTestRunEvent"""
+    """Fake :class:`~nose2.events.StartTestRunEvent`"""
 
     def __init__(self, runner=None, suite=None, result=None, startTime=None,
                  executeTests=None):
@@ -153,17 +183,23 @@ def run_nose2(*nose2_args, **nose2_kwargs):
         cwd = nose2_kwargs.pop('cwd')
         if not os.path.isabs(cwd):
             nose2_kwargs['cwd'] = support_file(cwd)
-    if 'module' not in nose2_kwargs:
-        nose2_kwargs['module'] = None
     return NotReallyAProc(nose2_args, **nose2_kwargs)
 
 
+def run_module_as_main(test_module, *args):
+    if not os.path.isabs(test_module):
+        test_module = support_file(test_module)
+    return subprocess.Popen([sys.executable, test_module] + list(args),
+                            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+
+
 class NotReallyAProc(object):
 
     def __init__(self, args, cwd=None, **kwargs):
         self.args = args
         self.chdir = cwd
         self.kwargs = kwargs
+        self.result = None
 
     def __enter__(self):
         self._stdout = sys.__stdout__
@@ -189,7 +225,7 @@ class NotReallyAProc(object):
                     argv=('nose2',) + self.args, exit=False,
                     **self.kwargs)
             except SystemExit as e:
-                return "", "EXIT CODE %s" % str(e)
+                pass
             return self.stdout.getvalue(), self.stderr.getvalue()
 
     @property
@@ -197,13 +233,15 @@ class NotReallyAProc(object):
         return id(self)
 
     def poll(self):
+        if self.result is None:
+            return 1
         return not self.result.result.wasSuccessful()
 
 
 class RedirectStdStreams(object):
 
     """
-    Context manager that replaces the stdin/out streams with StringIO
+    Context manager that replaces the stdin/stdout streams with :class:`StringIO`
     buffers.
     """
 
