@@ -22,6 +22,22 @@ from nose2.tests._common import FunctionalTestCase, support_file
 
 class TestLoadTestsFromPackage(FunctionalTestCase):
 
+    def test_start_directory_inside_package(self):
+        proc = self.runIn(
+            'scenario/tests_in_package/pkg1/test',
+            '-v',
+            '-t',
+            support_file('scenario/tests_in_package'))
+        self.assertTestRunOutputMatches(proc, stderr='Ran 25 tests')
+        self.assertEqual(proc.poll(), 1)
+
+    def test_project_directory_inside_package(self):
+        proc = self.runIn(
+            'scenario/tests_in_package/pkg1/test',
+            '-v')
+        self.assertTestRunOutputMatches(proc, stderr='Ran 25 tests')
+        self.assertEqual(proc.poll(), 1)
+
     def test_module_name(self):
         proc = self.runIn(
             'scenario/tests_in_package',
@@ -274,3 +290,21 @@ class TestTestClassLoading(FunctionalTestCase):
             'test_fixtures')
         self.assertTestRunOutputMatches(proc, stderr='Ran 5 tests')
         self.assertEqual(proc.poll(), 0)
+
+    def test_error_in_test_class(self):
+        proc = self.runIn(
+            'scenario/test_class_fail',
+            '-v',
+            'test_class_fail')
+        self.assertTestRunOutputMatches(proc, stderr='nose2.loader.LoadTestsFailure')
+        self.assertTestRunOutputMatches(proc, stderr='Ran 1 test')
+        self.assertTestRunOutputMatches(proc, stderr='FAILED')
+        self.assertEqual(proc.poll(), 1)
+    
+    def test_expected_failures(self):
+        proc = self.runIn(
+            'scenario/expected_failures',
+            '-v',
+            'expected_failures')
+        self.assertTestRunOutputMatches(proc, stderr=r'FAILED \(failures=1, expected failures=1, unexpected successes=1\)')
+        
\ No newline at end of file
