@@ -6,6 +6,11 @@ from nose2.plugins.mp import MultiProcess, procserver
 from nose2.plugins import buffer
 from nose2.plugins.loader import discovery, testcases
 from nose2.tests._common import FunctionalTestCase, support_file, Conn
+from six.moves import queue
+import multiprocessing
+import threading
+import time
+from multiprocessing import connection
 
 
 class TestMpPlugin(FunctionalTestCase):
@@ -71,6 +76,39 @@ class TestMpPlugin(FunctionalTestCase):
                                 'test_cf_testcase.Test3',
                                 ])
 
+    def test_conn_prep(self):
+        self.plugin.bind_host = None
+        (parent_conn, child_conn) = self.plugin._prepConns()
+        (parent_pipe, child_pipe) = multiprocessing.Pipe()
+        self.assertIsInstance(parent_conn, type(parent_pipe))
+        self.assertIsInstance(child_conn, type(child_pipe))
+
+        self.plugin.bind_host = "127.0.0.1"
+        self.plugin.bind_port = 0
+        (parent_conn, child_conn) = self.plugin._prepConns()
+        self.assertIsInstance(parent_conn, connection.Listener)
+        self.assertIsInstance(child_conn, tuple)
+        self.assertEqual(parent_conn.address, child_conn[:2])
+
+    def test_conn_accept(self):
+        (parent_conn, child_conn) = multiprocessing.Pipe()
+        self.assertEqual(self.plugin._acceptConns(parent_conn), parent_conn)
+
+        listener = connection.Listener(('127.0.0.1', 0))
+        with self.assertRaises(RuntimeError):
+            self.plugin._acceptConns(listener)
+
+        def fake_client(address):
+            client = connection.Client(address)
+            time.sleep(10)
+            client.close()
+
+        t = threading.Thread(target=fake_client, args=(listener.address,))
+        t.start()
+        conn = self.plugin._acceptConns(listener)
+        self.assertTrue(hasattr(conn, "send"))
+        self.assertTrue(hasattr(conn, "recv"))
+
 
 class TestProcserver(FunctionalTestCase):
 
@@ -164,3 +202,59 @@ class MPPluginTestRuns(FunctionalTestCase):
             '-N=2')
         self.assertTestRunOutputMatches(proc, stderr='Ran 7 tests')
         self.assertEqual(proc.poll(), 0)
+
+    def test_large_number_of_tests_stresstest(self):
+        proc = self.runIn(
+            'scenario/many_tests',
+            '-v',
+            '--plugin=nose2.plugins.mp',
+            '--plugin=nose2.plugins.loader.generators',
+            '-N=1')
+        self.assertTestRunOutputMatches(proc, stderr='Ran 600 tests')
+        self.assertEqual(proc.poll(), 0)
+
+    def test_socket_stresstest(self):
+        proc = self.runIn(
+            'scenario/many_tests_socket',
+            '-v',
+            '-c scenario/many_test_socket/nose2.cfg',
+            '--plugin=nose2.plugins.mp',
+            '--plugin=nose2.plugins.loader.generators',
+            '-N=1')
+        self.assertTestRunOutputMatches(proc, stderr='Ran 600 tests')
+        self.assertEqual(proc.poll(), 0)
+
+    def test_too_many_procs(self):
+        # Just need to run the mp plugin with less tests than
+        # processes.
+        proc = self.runModuleAsMain('scenario/one_test/tests.py',
+                                    '--log-level=debug',
+                                    '--plugin=nose2.plugins.mp',
+                                    '-N=2')
+        ret_vals = queue.Queue()
+
+        def save_return():
+            """
+            Popen.communciate() blocks.  Use a thread-safe queue
+            to return any exceptions.  Ideally, this completes
+            and returns None.
+            """
+            try:
+                self.assertTestRunOutputMatches(proc,
+                                                stderr='Ran 1 test')
+                self.assertEqual(proc.poll(), 0)
+                ret_vals.put(None)
+            except Exception as exc:
+                ret_vals.put(exc)
+        thread = threading.Thread(target=save_return)
+        thread.start()
+
+        # 1 minute should be more than sufficent for this
+        # little test case.
+        try:
+            exc = ret_vals.get(True, 60)
+        except queue.Empty:
+            exc = "MP Test timed out"
+            proc.kill()
+        self.assertIsNone(exc, str(exc))
+
