@@ -9,17 +9,19 @@ import os
 import re
 import sys
 import traceback
+import platform
 
 try:
-    from inspect import isgeneratorfunction # new in 2.6
+    from inspect import isgeneratorfunction  # new in 2.6
 except ImportError:
     import inspect
     try:
         from compiler.consts import CO_GENERATOR
     except ImportError:
         # IronPython doesn't have a complier module
-        CO_GENERATOR=0x20
+        CO_GENERATOR = 0x20
     # backported from Python 2.6
+
     def isgeneratorfunction(func):
         return bool((inspect.isfunction(func) or inspect.ismethod(func)) and
                     func.func_code.co_flags & CO_GENERATOR)
@@ -33,12 +35,12 @@ VALID_MODULE_RE = re.compile(r'[_a-zA-Z]\w*\.py$', re.UNICODE)
 
 
 def ln(label, char='-', width=70):
-    """Draw a divider, with label in the middle.
+    """Draw a divider, with ``label`` in the middle.
 
     >>> ln('hello there')
     '---------------------------- hello there -----------------------------'
 
-    Width and divider char may be specified. Defaults are 70 and '-'
+    ``width`` and divider ``char`` may be specified. Defaults are ``70`` and ``'-'``,
     respectively.
 
     """
@@ -52,12 +54,22 @@ def ln(label, char='-', width=70):
 
 
 def valid_module_name(path):
-    """Is path a valid module name?"""
+    """Is ``path`` a valid module name?"""
     return VALID_MODULE_RE.search(path)
 
 
 def name_from_path(path):
-    """Translate path into module name"""
+    """Translate ``path`` into module name
+
+    Returns a two-element tuple:
+
+    1. a dotted module name that can be used in an import statement
+       (e.g., ``pkg1.test.test_things``)
+
+    2. a full path to filesystem directory, which must be on ``sys.path``
+       for the import to succeed.
+
+    """
     # back up to find module root
     parts = []
     path = os.path.normpath(path)
@@ -70,21 +82,21 @@ def name_from_path(path):
             parts.append(top)
         else:
             break
-    return '.'.join(reversed(parts))
+    return '.'.join(reversed(parts)), candidate
 
 
 def module_from_name(name):
-    """Import module from name"""
+    """Import module from ``name``"""
     __import__(name)
     return sys.modules[name]
 
 
 def test_from_name(name, module):
-    """Import test from name"""
+    """Import test from ``name``"""
     pos = name.find(':')
     index = None
     if pos != -1:
-        real_name, digits = name[:pos], name[pos+1:]
+        real_name, digits = name[:pos], name[pos + 1:]
         try:
             index = int(digits)
         except ValueError:
@@ -96,7 +108,7 @@ def test_from_name(name, module):
 
 
 def object_from_name(name, module=None):
-    """Import object from name"""
+    """Import object from ``name``"""
     parts = name.split('.')
     if module is None:
         parts_copy = parts[:]
@@ -123,18 +135,25 @@ def name_from_args(name, index, args):
     return '%s:%s\n%s' % (name, index + 1, summary[:79])
 
 
-def test_name(test):
-    # XXX does not work for test funcs, test.id() lacks module
+def test_name(test, qualname=True):
+    # XXX does not work for test funcs; test.id() lacks module
     if hasattr(test, '_funcName'):
         tid = test._funcName
     elif hasattr(test, '_testFunc'):
         tid = "%s.%s" % (test._testFunc.__module__, test._testFunc.__name__)
     else:
-        tid = test.id()
+        if sys.version_info >= (3, 5) and not qualname:
+            test_module = test.__class__.__module__
+            test_class = test.__class__.__name__
+            test_method = test._testMethodName
+            tid = "%s.%s.%s" % (test_module, test_class, test_method)
+        else:
+            tid = test.id()
     if '\n' in tid:
         tid = tid.split('\n')[0]
     return tid
 
+
 def ispackage(path):
     """Is this path a package directory?"""
     if os.path.isdir(path):
@@ -152,13 +171,13 @@ def ispackage(path):
 
 
 def ensure_importable(dirname):
-    """Ensure a directory is on sys.path"""
-    if not dirname in sys.path:
+    """Ensure a directory is on ``sys.path``."""
+    if dirname not in sys.path:
         sys.path.insert(0, dirname)
 
 
 def isgenerator(obj):
-    """is this object a generator?"""
+    """Is this object a generator?"""
     return (isgeneratorfunction(obj)
             or getattr(obj, 'testGenerator', None) is not None)
 
@@ -174,8 +193,19 @@ def has_module_fixtures(test):
 
 
 def has_class_fixtures(test):
-    return ('setUpClass' in test.__class__.__dict__ or
-            'tearDownClass' in test.__class__.__dict__)
+    # hasattr would be the obvious thing to use here. Unfortunately, all tests
+    # inherit from unittest2.case.TestCase, and that *always* has setUpClass and
+    # tearDownClass methods. Thus, the following (ugly) solution:
+    ver = platform.python_version_tuple()
+    if float('{0}.{1}'.format(*ver[:2])) >= 2.7:
+        name = 'unittest.case'
+    else:
+        name = 'unittest2.case'
+    has_class_setups = any(
+        'setUpClass' in c.__dict__ for c in test.__class__.__mro__ if c.__module__.find(name) == -1)
+    has_class_teardowns = any(
+        'tearDownClass' in c.__dict__ for c in test.__class__.__mro__ if c.__module__.find(name) == -1)
+    return has_class_setups or has_class_teardowns
 
 
 def safe_decode(string):
@@ -204,7 +234,7 @@ def exc_info_to_string(err, test):
 
 
 def format_traceback(test, err):
-    """Converts a sys.exc_info()-style tuple of values into a string."""
+    """Converts a :func:`sys.exc_info` -style tuple of values into a string."""
     exctype, value, tb = err
     if not hasattr(tb, 'tb_next'):
         msgLines = tb
@@ -224,11 +254,11 @@ def format_traceback(test, err):
 
 
 def transplant_class(cls, module):
-    """Make class appear to reside in ``module``.
+    """Make ``cls`` appear to reside in ``module``.
 
     :param cls: A class
     :param module: A module name
-    :returns: A subclass of ``cls`` that appears to have been defined in ``module``. 
+    :returns: A subclass of ``cls`` that appears to have been defined in ``module``.
 
     The returned class's ``__name__`` will be equal to
     ``cls.__name__``, and its ``__module__`` equal to ``module``.
@@ -263,7 +293,9 @@ def _count_relevant_tb_levels(tb):
 
 
 class _WritelnDecorator(object):
-    """Used to decorate file-like objects with a handy 'writeln' method"""
+
+    """Used to decorate file-like objects with a handy :func:`writeln` method"""
+
     def __init__(self, stream):
         self.stream = stream
 
@@ -278,7 +310,8 @@ class _WritelnDecorator(object):
     def writeln(self, arg=None):
         if arg:
             self.stream.write(arg)
-        self.stream.write('\n') # text-mode streams translate to \r\n if needed
+        self.stream.write('\n')
+                          # text-mode streams translate to \r\n if needed
 
 
 def ancestry(layer):
@@ -299,4 +332,3 @@ def ancestry(layer):
 
 def bases_and_mixins(layer):
     return (layer.__bases__ + getattr(layer, 'mixins', ()))
-
